/*
CSV Import Utility
Licensed to users of N2CMS under the terms of the Boost Software License

Copyright (c) 2013 Benjamin Herila <mailto:ben@herila.net>

Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization obtaining a copy of the 
software and accompanying documentation covered by this license (the "Software") to use, reproduce,
display, distribute, execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to do so, all subject to 
the following:

The copyright notices in the Software and this entire statement, including the above license grant,
this restriction and the following disclaimer, must be included in all copies of the Software, in
whole or in part, and all derivative works of the Software, unless such copies or derivative works 
 are solely in the form of machine-executable object code generated by a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-
INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR
IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 * TODO: Add support for escape sequences
 * TODO: Add support for quoted strings and customizable quote characters

*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace N2.Extensions
{
    public class CsvImport
    {
        public CsvImport(ContentItem Model)
        {
            var dcStr = Model.GetDetail("Delimiter", ",");
            if (String.IsNullOrEmpty(dcStr))
                return; // can't do anything.
             
            var dc = (dcStr == "\\t" ? '\t' : dcStr[0]);
            var dd = Model.GetDetail("Data", string.Empty).Replace('\r', '\n').Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries);

            var bSkip = Model.GetDetail("Headers", false);
            var nSkip = bSkip ? 1 : 0;

            if (dd.Length - nSkip >= 0)
            {
                Rows = new string[Math.Max(dd.Length - nSkip, 0)][];
	            for (int i = nSkip; i < dd.Length && i < Rows.Length; ++i)
		            Rows[i] = SplitWithEscape(dd[i], dc);

                if (Model.GetDetail("Sort", false))
                    Rows = Rows.Where(f => f.Length > 0).OrderBy(f => f[0]).ToArray();
            }
            HasHeader = bSkip;
            if (bSkip)
                HeaderRow = dd[0].Split(dc);
        }

	    private static string[] SplitWithEscape(string str, char delimiter)
	    {
		    var items = new List<string>();
			var sb = new StringBuilder();
		    var escaped = false;
		    var esc = '\\';
		    foreach (var c in str)
		    {
			    if (c == esc)
			    {
				    if (escaped)
				    {
					    sb.Append(c);
					    escaped = false;
				    }
				    else
				    {
					    escaped = true;
				    }
			    }
				else if (c == delimiter)
				{
					if (escaped)
					{
						sb.Append(c);
						escaped = false;
					}
					else
					{
						items.Add(sb.ToString());
						sb.Clear();
					}
				}
				else
				{
					sb.Append(c);
				}
		    }
			if (sb.Length > 0)
				items.Add(sb.ToString());
		    return items.ToArray();
	    }


        public bool HasHeader { get; set; }
        public string[] HeaderRow { get; set; }
        public string[][] Rows { get; set; }

    }
}
